
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

const nsISupports = Components.interfaces.nsISupports;
//const nsIPrefswitch = Components.interfaces.nsIPrefswitch;
const CLASS_ID = Components.ID("{d98dd3e5-9060-450f-a788-a79cea05660c}");
const CLASS_NAME = "Prefswitch";
const CONTRACT_ID = "@prefswitch.mozdev.org;1";

const prefBranch = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);

var prefswitch = {
    DEFAULT_SWITCH_PATH: "extensions/prefswitch@mozdev.org/switch.xml",
    DEFAULT_PROFILE_PATH: "extensions/prefswitch@mozdev.org/profile.xml",
    DEFAULT_INJECT_PATH: "extensions/prefswitch@mozdev.org/inject",
    SWITCH_PATH: "prefswitch-switch.xml",
    PROFILE_PATH: "prefswitch-profile.xml",
    INJECT_PATH: "prefswitch-inject",

//    PROGRAM_NAME: "Prefswitch",

    BOOLEAN: "boolean",
    INTEGER: "integer",
    STRING: "string",
    RESET: "reset",
    LIST: "list",
    LISTITEMS: "listitems",

    UNKNOWN: 0,
    CSS: 1,
    JS: 2,

    LOAD_PROFILE: "loadProfile",
    TOGGLE_CSS: "toggleCSS",
    TOGGLE_JS: "toggleJS",

    ROOT: "extensions.prefswitch.",
    FIRSTRUN: "extensions.prefswitch.firstrun",
    VERSION: "extensions.prefswitch.version",
    UPDATE_ROOT: "extensions.prefswitch.update.",
    UPDATE: "extensions.prefswitch.update.value",
    PROFILE_ROOT: "extensions.prefswitch.profile.",
    PROFILE: "extensions.prefswitch.profile.value",
    SHOW: "extensions.prefswitch.show",
    SHOW_CSS_ROOT: "extensions.prefswitch.showCSS.",
    SHOW_CSS: "extensions.prefswitch.showCSS.value",
    SHOW_JS_ROOT: "extensions.prefswitch.showJS.",
    SHOW_JS: "extensions.prefswitch.showJS.value",
    VALUE: "value",

    CHARSET: "UTF-8",
    CHARACTERS: {
        "&#38;": /&/g,
        "&#34;": /\"/g,
        "&#60;": /</g,
        "&#62;": />/g
    }
}

prefswitch.setValue = function(preference, value, type)
{
    if (!preference || !type) return;

    try
    {
        switch(type)
        {
            case this.BOOLEAN:
                prefBranch.setBoolPref(preference, "" + value == "true");
                break;

            case this.INTEGER:
                prefBranch.setIntPref(preference, value);
                break;

            case this.STRING:
                prefBranch.setCharPref(preference, value);
                break;

            case this.RESET:
                prefBranch.clearUserPref(preference);
                break;

            case this.LIST:
                prefBranch.setCharPref(preference, value);
                break;

            case this.LISTITEMS:
                var p = this.getValue(preference, value, this.STRING);
                var a = value.split(" ");
                var t = "";
                for (var i = 0; i < a.length; i++)
                {
                    if (a[i] != "" && !p.match(new RegExp("(^|\\s)" + a[i].replace(/(\W)/g, "\\$1") + "(\\s|$)")))
                    {
                        if (t != "" || p != "") t += " ";
                        t += a[i];
                    }
                }
                if (t != "") prefBranch.setCharPref(preference, p + t);
                break;
        }
    }
    catch(e){}
}

prefswitch.getValue = function(preference, defaultValue, type)
{
    if (!preference || !type) return null;

    try
    {
        switch(type)
        {
            case this.BOOLEAN:
                return prefBranch.getBoolPref(preference);

            case this.INTEGER:
                return prefBranch.getIntPref(preference);

            case this.STRING:
            case this.LIST:
            case this.LISTITEMS:
                return prefBranch.getCharPref(preference);
        }
    }
    catch(e)
    {
        if (!defaultValue) return null;
        this.setValue(preference, defaultValue, type);
    }

    return this.getValue(preference, null, type);
}

prefswitch.deleteBranch = function(branch)
{
    if (!branch) return;

    prefBranch.deleteBranch(branch);
}

prefswitch.encode = function(string)
{
    if (!string) return "";

    for (var ch in this.CHARACTERS) string = string.replace(this.CHARACTERS[ch], ch);
    return string;
}

prefswitch.check = function()
{
    // FIRST RUN
    var firstrun = this.getValue(this.FIRSTRUN, true, this.BOOLEAN);
    if (firstrun)
    {
        this.setValue(this.FIRSTRUN, false, this.BOOLEAN);
    }

    // UPGRADE
    var version = this.getValue(this.VERSION, "", this.STRING);
    var extensionManager = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
    var current = extensionManager.getItemForID("prefswitch@mozdev.org").version;
    if (version != current && !firstrun)
    {
        this.setValue(this.VERSION, current, this.STRING);

        // INITIALIZE DEFAULT PROFILES AND SWITCHES
//        var file = this.getFile(this.DEFAULT_..);
//        if (!file /* profiles.xml or switches.xml is missing */)
//        {
//            var data = this.getData(file);
//            if (!data) return;
//            this.saveToFile(data, this.getFile(this.SWITCH_PATH));
//            this.saveSwitches(data);
//        }
    }
}

prefswitch.getSwitches = function(file)
{
    var doc = this.getDoc(file);
    if (!doc)
    {
        file = this.getFile(this.DEFAULT_SWITCH_PATH);
        var data = this.getData(file);
        this.saveSwitches(data);

        doc = this.getDoc(file);
        if (!doc) return [];
    }
    else if (doc.documentElement.nodeName == "parsererror")
    {
//        alert(this.PROGRAM_NAME + ": " + this.stringbundle.getString("ERROR_PARSING_SWITCHES"));
        return [];
    }

    var aResult = [];
    var iSwitch = doc.evaluate("./prefswitch/switch", doc, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
    var nSwitch = iSwitch.iterateNext();
    while (nSwitch)
    {
        var mySwitch = new PrefswitchSwitch(
            nSwitch.getAttribute("id"),
            nSwitch.getAttribute("name"),
            nSwitch.getAttribute("initial"));
        var iState = doc.evaluate("./state", nSwitch, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
        var nState = iState.iterateNext();
        while (nState)
        {
            var state = new PrefswitchState(
                nState.getAttribute("id"),
                nState.getAttribute("name"),
                nState.getAttribute("description"),
                nState.getAttribute("tooltip"),
                nState.getAttribute("color"));
            var iPreference = doc.evaluate("./preference", nState, null, Components.interfaces.nsIDOMXPathResult.ORDERED_NODE_ITERATOR_TYPE, null);
            var nPreference = iPreference.iterateNext();
            while (nPreference)
            {
                state.addPreference(new PrefswitchPreference(
                    nPreference.getAttribute("name"),
                    nPreference.getAttribute("type"),
                    nPreference.getAttribute("value"),
                    nPreference.getAttribute("description")));
                nPreference = iPreference.iterateNext();
            }
            mySwitch.addState(state);
            nState = iState.iterateNext();
        }
        aResult.push(mySwitch);
        nSwitch = iSwitch.iterateNext();
    }
    return aResult;
}

prefswitch.saveSwitches = function(data)
{
    if (!data) return;

    this.saveToFile(data, this.getFile(this.SWITCH_PATH));
}

prefswitch.getProfiles = function(file)
{
    var doc = this.getDoc(file);
    if (!doc)
    {
        file = this.getFile(this.DEFAULT_PROFILE_PATH);
        var data = this.getData(file);
        this.saveProfiles(data);

        doc = this.getDoc(file);
        if (!doc) return [];
    }
    else if (doc.documentElement.nodeName == "parsererror")
    {
//        alert(this.PROGRAM_NAME + ": " + this.stringbundle.getString("ERROR_PARSING_PROFILES"));
        return [];
    }

    var aResult = [];
    var iProfile = doc.evaluate("./prefswitch/profile", doc, null, Components.interfaces.nsIDOMXPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
    var nProfile = iProfile.iterateNext();
    while (nProfile)
    {
        var profile = new PrefswitchProfile(
            nProfile.getAttribute("id"),
            nProfile.getAttribute("name"));
        var iFeature = doc.evaluate("./feature", nProfile, null, Components.interfaces.nsIDOMXPathResult.UNORDERED_NODE_ITERATOR_TYPE, null);
        var nFeature = iFeature.iterateNext();
        while (nFeature)
        {
            profile.setFeature(nFeature.getAttribute("key"), nFeature.getAttribute("value"), nFeature.getAttribute("show"), nFeature.getAttribute("disabled"));
            nFeature = iFeature.iterateNext();
        }
        aResult.push(profile);
        nProfile = iProfile.iterateNext();
    }
    return aResult;    
}

prefswitch.saveProfiles = function(data)
{
    if (!data) return;

    this.saveToFile(data, this.getFile(this.PROFILE_PATH));
}

prefswitch.getDoc = function(file)
{
    if (!file) return null;

    var data = this.getData(file);
    if (!data || data == "") return null;

    var parser = Components.classes['@mozilla.org/xmlextras/domparser;1'].createInstance(Components.interfaces.nsIDOMParser);
    return parser.parseFromString(data, "text/xml");
}

prefswitch.getDirectory = function(path)
{
    if (!path) return null;

    var directory = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
    var aPath = path.split(/\/|\\/);
    for (var i = 0; i < aPath.length; i++) directory.append(aPath[i]);
    if (!directory.exists()) return null;
    if (directory.isDirectory()) return directory;
    else return null;
}

prefswitch.getInjectDirectory = function(path)
{
    if (!path) return null;

    var created = false;

    var directory = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
    var aPath = path.split(/\/|\\/);
    for (var i = 0; i < aPath.length; i++) directory.append(aPath[i]);
    if (!directory.exists())
    {
        directory.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0700);
        created = true;
    }
    if (directory.isDirectory())
    {
        if (created)
        {
            var injectDirectory = this.getDirectory(this.DEFAULT_INJECT_PATH);
            if (!injectDirectory) return directory

            var entries = injectDirectory.directoryEntries;
            while (entries.hasMoreElements())
            {
                var entry = entries.getNext();
                entry.QueryInterface(Components.interfaces.nsIFile);
                if (entry.isFile() && /(\.css|\.js|\-ChangeLog)$/i.test(entry.leafName))
                {
                    var tmpFile = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
                    for (var i = 0; i < aPath.length; i++) tmpFile.append(aPath[i]);
                    tmpFile.append(entry.leafName);
                    var data = this.getData(entry);
                    this.saveToFile(data, tmpFile);
                }
            }
        }
        return directory;
    }
    else return null;
}

prefswitch.getData = function(file)
{
    if (!file) return "";

    var fis = Components.classes['@mozilla.org/network/file-input-stream;1'].createInstance(Components.interfaces.nsIFileInputStream);
    fis.init(file, 0x01, 00004, null);
    var sis = Components.classes['@mozilla.org/scriptableinputstream;1'].createInstance(Components.interfaces.nsIScriptableInputStream);
    sis.init(fis);
    var data = sis.read(sis.available());
//    var suc = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
//    suc.charset = this.CHARSET;
//    var data = suc.ConvertToUnicode(data);
    sis.close();
    fis.close();
    return data;
}

prefswitch.getFile = function(path, noCreate)
{
    if (!path) return null;

    var file = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties).get("ProfD", Components.interfaces.nsIFile);
    var aPath = path.split(/\/|\\/);
    for (var i = 0; i < aPath.length; i++) file.append(aPath[i]);
    if (!noCreate && !file.exists()) file.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0600);
    if (file.exists() && file.isFile()) return file;
    else return null;
}

prefswitch.saveToFile = function(data, file)
{
    if (!data || !file) return;

    var fos = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
    fos.init(file, 0x02 | 0x08 | 0x20, 0600, 0);
    fos.write(data, data.length);
    fos.close();
}

prefswitch.QueryInterface = function(aIID)
{
    if (/*!aIID.equals(nsIPrefswitch) &&*/ !aIID.equals(nsISupports)) throw Components.results.NS_ERROR_NO_INTERFACE;
    return this;
}

prefswitch.wrappedJSObject = prefswitch;

var factory = {
    createInstance: function (aOuter, aIID)
    {
        if (aOuter != null) throw Components.results.NS_ERROR_NO_AGGREGATION;

        var scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].getService(Components.interfaces.mozIJSSubScriptLoader);
        scriptLoader.loadSubScript("chrome://prefswitch/content/switch.js");
        scriptLoader.loadSubScript("chrome://prefswitch/content/state.js");
        scriptLoader.loadSubScript("chrome://prefswitch/content/preference.js");
        scriptLoader.loadSubScript("chrome://prefswitch/content/profile.js");

        return prefswitch;
    }
}

var module = {
    registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
    {
        aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
    },

    unregisterSelf: function(aCompMgr, aLocation, aType)
    {
        aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);
    },

    getClassObject: function(aCompMgr, aCID, aIID)
    {
        if (!aIID.equals(Components.interfaces.nsIFactory)) throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        if (aCID.equals(CLASS_ID)) return factory;
        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    canUnload: function(aCompMgr) { return true; }
}

function NSGetModule(aCompMgr, aFileSpec) { return module; }
