
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function PrefswitchOverlay()
{
    this.updateAllObserver = null;
    this.profileObserver = null;
    this.cssObserver = null;
    this.jsObserver = null;

    this.appcontent = null;
    this.statusbar = null;
    this.popupset = null;
    this.profilePopup = null;
    this.showHideMenupopup = null;
    this.showHideSeparator = null;
    this.cssStatusbarpanel = null;
    this.cssPopup = null;
    this.cssMenuitem = null;
    this.jsStatusbarpanel = null;
    this.jsPopup = null;
    this.jsMenuitem = null;

    this.jsObject = {}

    this.openDialog = function(dialogName)
    {
        if (!dialogName) return;

        switch(dialogName)
        {
            case "profile":
                window.openDialog(
                    "chrome://prefswitch/content/profiledialog.xul",
                    "profiledialog",
                    "chrome,modal,resizable",
                    prefswitch,
                    this.helper/*,
                    profileName*/);
                break;

            case "switch":
                window.openDialog(
                    "chrome://prefswitch/content/switchdialog.xul",
                    "switchdialog",
                    "chrome,resizable",
                    prefswitch,
                    this.helper,
                    document.popupNode.sw ? document.popupNode.sw.getName() : null/*,
                    stateName*/);
                break;

            default: return;
        }
    }

    this.toggleCSS = function(show)
    {
        if (show != true && show != false) show = !prefswitch.getValue(prefswitch.SHOW_CSS, true, prefswitch.BOOLEAN);
        prefswitch.setValue(prefswitch.SHOW_CSS, show, prefswitch.BOOLEAN);

        this.cssStatusbarpanel.setAttribute("hidden", "" + !show);
        this.cssMenuitem.setAttribute("checked", "" + show);
    }

    this.toggleJS = function(show)
    {
        if (show != true && show != false) show = !prefswitch.getValue(prefswitch.SHOW_JS, true, prefswitch.BOOLEAN);
        prefswitch.setValue(prefswitch.SHOW_JS, show, prefswitch.BOOLEAN);

        this.jsStatusbarpanel.setAttribute("hidden", "" + !show);
        this.jsMenuitem.setAttribute("checked", "" + show);
    }

    this.openHelp = function(url, topic, contentPackSpec)
    {
        if (this.helper) this.helper.openHelp(url, topic, contentPackSpec);
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.type)
        {
            case "load":
                this.updateAllObserver = new PrefswitchObserver(
                    prefswitch.UPDATE_ROOT,
                    prefswitch.VALUE,
                    function() { prefswitchOverlay.updateAll(); },
                    function(time) { prefswitch.setValue(prefswitch.UPDATE, time, prefswitch.STRING); }
                );
                this.profileObserver = new PrefswitchObserver(
                    prefswitch.PROFILE_ROOT,
                    prefswitch.VALUE,
                    function() { var menuitem = prefswitchOverlay.updateProfiles(); prefswitchOverlay.loadProfile(menuitem, false); },
                    function(profileID) { prefswitch.setValue(prefswitch.PROFILE, profileID, prefswitch.STRING); }
                );
                this.cssObserver = new PrefswitchObserver(
                    prefswitch.SHOW_CSS_ROOT,
                    prefswitch.VALUE,
                    function() { prefswitchOverlay.toggleCSS(prefswitch.getValue(prefswitch.SHOW_CSS, true, prefswitch.BOOLEAN)); }
                );
                this.jsObserver = new PrefswitchObserver(
                    prefswitch.SHOW_JS_ROOT,
                    prefswitch.VALUE,
                    function() { prefswitchOverlay.toggleJS(prefswitch.getValue(prefswitch.SHOW_JS, true, prefswitch.BOOLEAN)); }
                );

                this.appcontent = document.getElementById("appcontent");
                this.statusbar = document.getElementById("status-bar");
                this.popupset = document.getElementById("prefswitch-popupset");
                this.profilePopup = document.getElementById("prefswitch-profile-popup");
                this.showHideMenupopup = document.getElementById("prefswitch-showHide-menupopup");
                this.showHideSeparator = document.getElementById("prefswitch-showHide-separator");
                this.cssStatusbarpanel = document.getElementById("prefswitch-css-statusbarpanel");
                this.cssPopup = document.getElementById("prefswitch-css-popup");
                this.cssMenuitem = document.getElementById("prefswitch-css-menuitem");
                this.jsStatusbarpanel = document.getElementById("prefswitch-js-statusbarpanel");
                this.jsPopup = document.getElementById("prefswitch-js-popup");
                this.jsMenuitem = document.getElementById("prefswitch-js-menuitem");

//                prefswitch.stringbundle = document.getElementById("prefswitch-overlay-stringbundle");

                this.toggleCSS(prefswitch.getValue(prefswitch.SHOW_CSS, true, prefswitch.BOOLEAN));
                this.toggleJS(prefswitch.getValue(prefswitch.SHOW_JS, true, prefswitch.BOOLEAN));

                this.checkFirstRun();
                this.updateAll();

                if ("openHelp" in window) this.injectCSS("chrome://prefswitch/content/help.css", true);
                this.helper = new Helper(window, gBrowser);

                if (this.appcontent) this.appcontent.addEventListener("DOMContentLoaded", this, false);
                window.removeEventListener("load", this, false);

                break;

            case "unload":
                this.removePopups();
                this.removeStatusbarpanels();
                this.removeMenuitems();

                this.updateAllObserver.unregister();
                this.profileObserver.unregister();
                this.cssObserver.unregister();
                this.jsObserver.unregister();

                if (this.appcontent) this.appcontent.removeEventListener("DOMContentLoaded", this, false);
                window.removeEventListener("unload", this, false);

                break;

            case "DOMContentLoaded":
                this.injectJavaScript(event.target.defaultView);
        }
    }

    this.checkFirstRun = function()
    {
        // FIRST RUN
        var firstrun = prefswitch.getValue(prefswitch.FIRSTRUN, true, prefswitch.BOOLEAN);
        if (firstrun)
        {
            prefswitch.setValue(prefswitch.FIRSTRUN, false, prefswitch.BOOLEAN);
        }

        // UPGRADE
        var version = prefswitch.getValue(prefswitch.VERSION, "", prefswitch.STRING);
        var extensionManager = Components.classes["@mozilla.org/extensions/manager;1"].getService(Components.interfaces.nsIExtensionManager);
        var current = extensionManager.getItemForID("prefswitch@mozdev.org").version;
        if (version != current && !firstrun)
        {
            prefswitch.setValue(prefswitch.VERSION, current, prefswitch.STRING);
        }
    }

    this.updateAll = function()
    {
        var menuitem = this.updateProfiles();

        this.removePopups();
        this.removeStatusbarpanels();
        this.removeMenuitems();

        this.addStatusbarpanels();
        this.addMenuitems();

        this.loadProfile(menuitem);
    }

    this.cleanup = function()
    {
        // BACKUP
        var firstrun = prefswitch.getValue(prefswitch.FIRSTRUN, null, prefswitch.BOOLEAN);
        var version = prefswitch.getValue(prefswitch.VERSION, null, prefswitch.STRING);
        var profile = prefswitch.getValue(prefswitch.PROFILE, "default", prefswitch.STRING);
        var show = prefswitch.getValue(prefswitch.SHOW, null, prefswitch.STRING);
        var showCSS = prefswitch.getValue(prefswitch.SHOW_CSS, true, prefswitch.BOOLEAN);
        var showJS = prefswitch.getValue(prefswitch.SHOW_JS, true, prefswitch.BOOLEAN);
        var arr = [this.cssPopup.firstChild, this.jsPopup.firstChild];
        var obj = {}
        for (var i = 0; i < arr.length; i++)
        {
            var m = arr[i];
            while (m)
            {
                var key = m.sw.getID() + "." + prefswitch.VALUE;
                obj[key] = prefswitch.getValue(prefswitch.ROOT + key, null, prefswitch.STRING);
                m = m.nextSibling;
            }
        }
        var sp = this.cssStatusbarpanel.previousSibling;
        while (sp.getAttribute("class") == "prefswitch-initial")
        {
            var key = sp.sw.getID() + "." + prefswitch.VALUE;
            obj[key] = prefswitch.getValue(prefswitch.ROOT + key, null, prefswitch.STRING);
            sp = sp.previousSibling;
        }

        // CLEANUP
        prefswitch.deleteBranch(prefswitch.ROOT);

        // RESTORE
        for (var key in obj) prefswitch.setValue(prefswitch.ROOT + key, obj[key], prefswitch.STRING);
        prefswitch.setValue(prefswitch.FIRSTRUN, firstrun, prefswitch.BOOLEAN);
        prefswitch.setValue(prefswitch.VERSION, version, prefswitch.STRING);
        prefswitch.setValue(prefswitch.PROFILE, profile, prefswitch.STRING);
        prefswitch.setValue(prefswitch.SHOW, show, prefswitch.STRING);
        prefswitch.setValue(prefswitch.SHOW_CSS, showCSS, prefswitch.BOOLEAN);
        prefswitch.setValue(prefswitch.SHOW_JS, showJS, prefswitch.BOOLEAN);

        this.updateAll();
        this.updateAllObserver.notify((new Date()).getTime());
    }

    this.removeStatusbarpanels = function()
    {
        var element = this.cssStatusbarpanel.previousSibling;
        while (element.getAttribute("class") == "prefswitch-initial")
        {
            var next = element.previousSibling;
            element.ob.unregister();
            this.statusbar.removeChild(element);
            element = next;
        }
    }

    this.removePopups = function()
    {
        var popup = this.popupset.firstChild;
        while (popup)
        {
            var next = popup.nextSibling;
            if (popup.tagName == "popup") this.popupset.removeChild(popup);
            popup = next;
        }
    }

    this.removeMenuitems = function()
    {
        var arr = [this.cssPopup, this.jsPopup];
        for (var i = 0; i < arr.length; i++)
        {
            var menuitem = arr[i].firstChild;
            while (menuitem)
            {
                var next = menuitem.nextSibling;
                menuitem.ob.unregister();
                arr[i].removeChild(menuitem);
                menuitem = next;
            }
        }
    }

    this.addStatusbarpanels = function()
    {
        // REMOVE SHOW-HIDE-MENUITEMS
        while (this.showHideSeparator.previousSibling) this.showHideMenupopup.removeChild(this.showHideSeparator.previousSibling);
        var file = prefswitch.getFile(prefswitch.SWITCH_PATH, true);
        var aSwitch = prefswitch.getSwitches(file);

        // CREATE STATUSBARPANELS
        while (aSwitch.length > 0)
        {
            var sw = aSwitch.shift();
            if (sw.isHidden()) continue;
            var statusbarpanel = this.createSwitchElement(sw, "statusbarpanel", "prefswitch-initial", sw.getInitial(), "onclick");
            if (!statusbarpanel) continue;
            statusbarpanel.setAttribute("hidden", "true");
            statusbarpanel.setAttribute("context", "prefswitch-menupopup");
            this.statusbar.insertBefore(statusbarpanel, this.cssStatusbarpanel);

            // CREATE SHOW-HIDE-MENUITEMS
            var menuitem = document.createElement("menuitem");
            menuitem.setAttribute("type", "checkbox");
            menuitem.setAttribute("label", sw.getName());
            statusbarpanel.mi = menuitem;
            menuitem.sp = statusbarpanel;
            menuitem.setAttribute("oncommand", "prefswitchOverlay.hideShow(this)");
            this.showHideMenupopup.insertBefore(menuitem, this.showHideSeparator);
        }
    }

    this.hideShow = function(menuitem)
    {
        var hidden = !menuitem.getAttribute('checked');
        var statusbarpanel = menuitem.sp;
        if (hidden) this.removeShow(statusbarpanel.sw.getID());
        else this.addShow(statusbarpanel.sw.getID());

        statusbarpanel.setAttribute("hidden", hidden);
        prefswitch.setValue(prefswitch.PROFILE, "user-" + (new Date()).getTime(), prefswitch.STRING);
    }

    this.addMenuitems = function()
    {
        // GET FILES
        var aFiles = [];
        var directory = prefswitch.getInjectDirectory(prefswitch.INJECT_PATH);
        if (directory)
        {
            var entries = directory.directoryEntries;
            while (entries.hasMoreElements())
            {
                var entry = entries.getNext();
                entry.QueryInterface(Components.interfaces.nsIFile);
                if (!entry.isFile() || !entry.leafName.match(/\.(css|js)$/i)) continue;
                aFiles.push([entry.leafName.replace(/\./g, "_"), entry.leafName.replace(/^\d+/, "").replace(/\.\w+$/, ""), entry.path]);
            }
        }

        // SORT FILES
        aFiles.sort();

        // CREATE MENUITEMS
        while (aFiles.length > 0)
        {
            var file = aFiles.shift();
            var sw = new PrefswitchSwitch(file[0], file[1], "-");
            sw.setInject(true);
            sw.setInjectPath(file[2]);

            var disabledState = new PrefswitchState("extract-" + file[1], "extract-" + file[1]);
            disabledState.addPreference(new PrefswitchPreference(prefswitch.ROOT + file[0] + "." + prefswitch.VALUE, prefswitch.BOOLEAN, false));
            sw.addState(disabledState);
            var enabledState = new PrefswitchState("inject-" + file[1], "inject-" + file[1]);
            enabledState.addPreference(new PrefswitchPreference(prefswitch.ROOT + file[0] + "." + prefswitch.VALUE, prefswitch.BOOLEAN, true));
            sw.addState(enabledState);

            var menuitem = this.createSwitchElement(sw, "menuitem", null, sw.getName(), "oncommand");
            if (!menuitem) continue;
            menuitem.setAttribute("type", "checkbox");
            switch(sw.getInjectType())
            {
                case prefswitch.CSS:
                    this.cssPopup.appendChild(menuitem);
                    break;

                case prefswitch.JS:
                    this.jsPopup.appendChild(menuitem);
            }
        }
    }

    this.createSwitchElement = function(sw, elementType, className, labelName, eventName)
    {
        if (!sw || !elementType || !labelName || !eventName) return null;

        var element = document.createElement(elementType);
        element.sw = sw;
        element.ob = new PrefswitchObserver(
            prefswitch.ROOT + sw.getID() + ".",
            prefswitch.VALUE,
            function() { prefswitchOverlay.updateOne(element); }
        );
        if (className) element.setAttribute("class", className);
        element.setAttribute("label", labelName);

        // APPEND CLICK EVENT OR CREATE POPUP MENU
        if (elementType != "statusbarpanel" && sw.isBooleanSwitch()) element.setAttribute(eventName, "prefswitchOverlay.onclick(event, this.sw, null, true)");
        else
        {
            var popup = document.createElement("popup");
            var id = "prefswitch-popup-" + sw.getID();
            popup.setAttribute("id", id);

            // ADD MENUITEMS
            for (var i = 0; i < sw.getStates().length; i++)
            {
                var st = sw.getState(i);
                var menuitem = document.createElement("menuitem");
                menuitem.setAttribute("type", "radio");
                menuitem.setAttribute("name", "prefswitch-radio-" + sw.getID());
                menuitem.setAttribute("label", st.getName());
                menuitem.sw = sw;
                menuitem.st = st;
                st.menuitem = menuitem;
                menuitem.setAttribute("oncommand", "prefswitchOverlay.onclick(event, this.sw, this.st, false)");
                popup.appendChild(menuitem);
            }
            this.popupset.appendChild(popup);
            element.setAttribute("popup", id);
        }

        this.updateOne(element);
        return element;
    }

    this.updateOne = function(element)
    {
        if (!element) return;

        var sw = element.sw;
        if (!sw) return;

        var value = prefswitch.getValue(prefswitch.ROOT + sw.getID() + "." + prefswitch.VALUE, sw.getState(0).getID(), prefswitch.STRING);
        var st = sw.getStateByID(value);
        if (!st) return;

        switch(element.tagName)
        {
            case "statusbarpanel":
                element.setAttribute("style", "color:" + st.getColor());
                var tooltiptext = st.getTooltip();
                if (tooltiptext != "") element.setAttribute("tooltiptext", tooltiptext);
                if (st.menuitem)
                {
                    var menuitem = st.menuitem.parentNode.firstChild;
                    while (menuitem)
                    {
                        if (st.menuitem == menuitem) menuitem.setAttribute("checked", "true");
                        else menuitem.setAttribute("checked", "false");
                        menuitem = menuitem.nextSibling;
                    }
                }
                break;

            case "menuitem":
                var path = sw.getInjectPath();
                var inject = st.getPreference(0).getValue() == "true";
                switch(sw.getInjectType())
                {
                    case prefswitch.CSS:
                        this.injectCSS("file://" + path, inject);
                        break;

                    case prefswitch.JS: this.setJavascript(sw.getID(), path, inject);
                        break;

                    default: return;
                }
                element.setAttribute("checked", "" + inject);
        }
    }

    this.injectCSS = function(path, value)
    {
        if (!path) return;

        var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
        var uri = ios.newURI(path, prefswitch.CHARSET, null); // URI, CHARSET, BASEURI
        var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"].getService(Components.interfaces.nsIStyleSheetService);
        if (value) { if (!sss.sheetRegistered(uri, sss.USER_SHEET)) sss.loadAndRegisterSheet(uri, sss.USER_SHEET); }
        else { if (sss.sheetRegistered(uri, sss.USER_SHEET)) sss.unregisterSheet(uri, sss.USER_SHEET); }
    }

    this.setJavascript = function(key, path, value)
    {
        if (!key || !path) return;

        if (value) this.jsObject[key] = path;
        else this.jsObject[key] = null;
    }

    this.injectJavaScript = function(defaultView)
    {
        if (!defaultView || defaultView.location.protocol.match(/^about:$/i)) return;

        for (var key in this.jsObject)
        {
            var path = this.jsObject[key];
            if (!path) continue;
            var file = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
            file.initWithPath(path);
            var lineFinder = null;
            try{
                var data = prefswitch.getData(file);
                var safeView = new XPCNativeWrapper(defaultView);
                var sandbox = Components.utils.Sandbox(safeView);
                sandbox.window = safeView;
                sandbox.document = safeView.document;
                sandbox.XPathResult = Components.interfaces.nsIDOMXPathResult;
                sandbox.__proto__ = safeView;
                lineFinder = new Error();
                Components.utils.evalInSandbox(data, sandbox);
            }catch(e){
                if (!lineFinder) throw e;
                else
                {
                        var consoleService = Components.classes['@mozilla.org/consoleservice;1'].getService(Components.interfaces.nsIConsoleService);
                    var scriptError = Components.classes['@mozilla.org/scripterror;1'].createInstance(Components.interfaces.nsIScriptError);
                    var sourceName = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService).newFileURI(file).spec;
                    scriptError.init(e.message, sourceName, null, e.lineNumber - lineFinder.lineNumber, e.columnNumber, scriptError.errorFlag, "content javascript");
                    consoleService.logMessage(scriptError);
                }
            }
        }
    }

    this.onclick = function(event, sw, st, isBooleanSwitch)
    {
        if (!event || !sw) return;
        if (event.button && event.button > 0) return;

        var switchID = sw.getID();

        if (isBooleanSwitch)
        {
            st = sw.getState(0);
            var stateID = st.getID();
            var value = prefswitch.getValue(prefswitch.ROOT + switchID + "." + prefswitch.VALUE, stateID, prefswitch.STRING);
            if (value == stateID) st = sw.getState(1);
        }

        if (!st) return;

        for (var i = 0; i < st.getPreferences().length; i++)
        {
            var pr = st.getPreference(i);
            if (pr.isHidden()) continue;
            prefswitch.setValue(pr.getName(), pr.getValue(), pr.getType());
        }

        prefswitch.setValue(prefswitch.ROOT + switchID + "." + prefswitch.VALUE, st.getID(), prefswitch.STRING);

        if (event != prefswitch.LOAD_PROFILE && !sw.getInject()) prefswitch.setValue(prefswitch.PROFILE, "user-" + (new Date()).getTime(), prefswitch.STRING);
    }

    this.updateProfiles = function()
    {
        while (this.profilePopup.firstChild) this.profilePopup.removeChild(this.profilePopup.firstChild);

        var profileID = prefswitch.getValue(prefswitch.PROFILE, null, prefswitch.STRING);
        var current = null;
        var file = prefswitch.getFile(prefswitch.PROFILE_PATH, true);
        var aProfile = prefswitch.getProfiles(file);
        while (aProfile.length > 0)
        {
            var profile = aProfile.shift();
            var menuitem = document.createElement("menuitem");
            menuitem.setAttribute("type", "radio");
            menuitem.setAttribute("name", "prefswitch-profile-menuitem");
            menuitem.setAttribute("label", profile.getName());
            menuitem.profile = profile;
            menuitem.setAttribute("oncommand", "prefswitchOverlay.loadProfile(this, true)");
            if (profileID == profile.getID())
            {
                menuitem.setAttribute("checked", "true");
                current = menuitem;
            }
            this.profilePopup.appendChild(menuitem);
        }

        return current;
    }

    this.addShow = function(list, clear)
    {
        if (!list || list == "") return;

        var obj = {}
        var arr = null;

        if (!clear)
        {
            var show = prefswitch.getValue(prefswitch.SHOW, "", prefswitch.STRING);
            if (show)
            {
                arr = show.split(/\s+/);
                for (var i = 0; i < arr.length; i++) obj[arr[i]] = true;
            }
        }

        arr = list.split(/\s+/);
        for (var i = 0; i < arr.length; i++) obj[arr[i]] = true;

        var show = "";
        for (var key in obj) show += key + " ";

        prefswitch.setValue(prefswitch.SHOW, show.replace(/\s+$/, ""), prefswitch.STRING);
    }

    this.removeShow = function(list)
    {
        if (!list || list == "") return;

        var obj = {}
        var arr = null;

        var show = prefswitch.getValue(prefswitch.SHOW, "", prefswitch.STRING);
        if (show)
        {
            arr = show.split(/\s+/);
            for (var i = 0; i < arr.length; i++) obj[arr[i]] = true;
        }

        arr = list.split(/\s+/);
        for (var i = 0; i < arr.length; i++) obj[arr[i]] = false;

        show = "";
        for (var key in obj) if (obj[key]) show += key + " ";

        prefswitch.setValue(prefswitch.SHOW, show, prefswitch.STRING);
    }

    this.loadProfile = function(menuitem)
    {
        if (!menuitem || !menuitem.profile)
        {
            var visible = prefswitch.getValue(prefswitch.SHOW, "", prefswitch.STRING);

            var obj = {}
            if (visible)
            {
                var arr = visible.split(/\s+/);
                for (var i = 0; i < arr.length; i++) obj[arr[i]] = true;
            }

            var statusbarpanel = this.cssStatusbarpanel.previousSibling;
            while (statusbarpanel.getAttribute("class") == "prefswitch-initial")
            {
                var sw = statusbarpanel.sw;
                var show = obj[sw.getID()] ? obj[sw.getID()] == true : false;
                statusbarpanel.setAttribute("hidden", "" + !show);
                statusbarpanel.mi.setAttribute("checked", "" + show);
                statusbarpanel = statusbarpanel.previousSibling;
            }

            return;
        }

        var visible = "";

        var profile = menuitem.profile;
        var statusbarpanel = this.cssStatusbarpanel.previousSibling;
        while (statusbarpanel.getAttribute("class") == "prefswitch-initial")
        {
            var sw = statusbarpanel.sw;
            var feature = profile.getFeature(sw.getID());

            if (feature)
            {
//                var st = sw.getStateByID(feature.value);
                statusbarpanel.setAttribute("hidden", "" + !feature.show);
                statusbarpanel.mi.setAttribute("checked", "" + feature.show);

                if (feature.show) visible += sw.getID() + " ";

//                if (!feature.disabled) this.onclick(prefswitch.LOAD_PROFILE, sw, st, false);
                if (!feature.disabled) this.onclick(prefswitch.LOAD_PROFILE, sw, sw.getStateByID(feature.value), false);
            }
            else
            {
                statusbarpanel.setAttribute("hidden", "true");
                statusbarpanel.mi.setAttribute("checked", "false");
            }

            statusbarpanel = statusbarpanel.previousSibling;
        }

        this.addShow(visible, true);
        this.profileObserver.notify(profile.getID());
    }
}
