
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function PrefswitchProfileDialog()
{
    this.listbox = null;
    this.dialogheader = null;
    this.vbox = null;
    this.listitem = null;
    this.profile = null;

    this.click = function(switchID, stateID, show, disabled)
    {
        if (!this.profile) return;

        this.profile.setFeature(switchID, stateID, show, disabled);
    }

    this.disableFeature = function(switchID)
    {
        if (!switchID) return;
        if (!this.profile) return;

        var feature = this.profile.getFeature(switchID);
        feature.disabled = !(document.getElementById("disabled-" + switchID).getAttribute("checked") == "true");
//        document.getElementById("show-" + switchID).setAttribute("disabled", feature.disabled);
        document.getElementById(switchID).setAttribute("disabled", feature.disabled);
    }

    this.moveProfileUp = function()
    {
        var index = this.listbox.selectedIndex;
        if (index > 0)
        {
            this.listbox.removeEventListener("select", this, false);
            var profile = this.listbox.removeItemAt(index).profile;
            var listitem = this.listbox.insertItemAt(index - 1, profile.getName(), "");
            listitem.profile = profile;
            this.listbox.selectedIndex = index - 1;
            this.listbox.addEventListener("select", this, false);
        }
    }

    this.moveProfileDown = function()
    {
        var index = this.listbox.selectedIndex;
        if (index > -1 && index < this.listbox.getRowCount() - 1)
        {
            this.listbox.removeEventListener("select", this, false);
            var profile = this.listbox.removeItemAt(index).profile;
            var listitem = this.listbox.insertItemAt(index + 1, profile.getName(), "");
            listitem.profile = profile;
            this.listbox.selectedIndex = index + 1;
            this.listbox.addEventListener("select", this, false);
        }
    }

    this.newProfile = function()
    {
        this.profile = new PrefswitchProfile(null, "New Profile");
        var listitem = this.listbox.appendItem(this.profile.getName());
        listitem.profile = this.profile;
        this.listbox.selectedIndex = this.listbox.getRowCount() - 1;
        this.refreshProfiles();
        if (!this.renameProfile())
        {
            this.listbox.removeItemAt(this.listbox.selectedIndex);
            if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
            else this.refreshProfiles();
        }
    }

    this.renameProfile = function()
    {
        if (this.profile == null) return false;
        var name = prompt("Enter name of profile:", this.profile.getName());
        if (name == null || name == "") return false;
        this.profile.setName(name);
        this.refreshProfiles();
        return true;
    }

    this.removeProfile = function()
    {
        if (this.listbox.selectedIndex < 0) return;
        this.listbox.removeItemAt(this.listbox.selectedIndex);
        if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
    }

    this.openHelp = function(url, topic, contentPackSpec)
    {
        if (this.helper) this.helper.openHelp(url, topic, contentPackSpec);
    }

    this.load = function()
    {
        var profileName = null;
        if ("arguments" in window && window.arguments.length > 0)
        {
            if (!prefswitch) prefswitch = window.arguments[0];
            this.helper = window.arguments[1];
            profileName = window.arguments[2];
        }

        this.listbox = document.getElementById("listbox");
        this.dialogheader = document.getElementById("dialogheader");
        this.vbox = document.getElementById("vbox");
        this.listbox.addEventListener("select", this, false);
        this.loadMenulists();
        this.loadProfiles(profileName);
        if (this.listbox.getRowCount() > 0) this.listbox.selectedIndex = 0;
    }

    this.handleEvent = function(event)
    {
        this.refreshProfiles();
    }

    this.loadMenulists = function()
    {
        var file = prefswitch.getFile(prefswitch.SWITCH_PATH, true);
        var aSwitch = prefswitch.getSwitches(file);
        var mySwitch = aSwitch.shift();
        while (mySwitch)
        {
            if (mySwitch.isHidden())
            {
                mySwitch = aSwitch.shift();
                continue;
            }
            var switchID = mySwitch.getID();
            var hbox = document.createElement("hbox");
            hbox.setAttribute("align", "center");

            // CREATE SHOW/HIDE MENULIST
            var menulist = document.createElement("menulist");
            menulist.setAttribute("id", "show-" + switchID);
            menulist.setAttribute("name", switchID);
            var menupopup = document.createElement("menupopup");
            var menuitem = document.createElement("menuitem");

            var stringbundle = document.getElementById("prefswitch-profile-stringbundle");

            menuitem.setAttribute("label", stringbundle.getString("menuitemShow"));
            menuitem.setAttribute("value", "true");
            menuitem.setAttribute(
                "oncommand",
                "prefswitchProfileDialog.click('" + switchID + "', document.getElementById('" + switchID + "').selectedItem.getAttribute('id'), true, document.getElementById('disabled-" + switchID + "').getAttribute('checked') != 'true')");
            menupopup.appendChild(menuitem);
            menuitem = document.createElement("menuitem");
            menuitem.setAttribute("label", stringbundle.getString("menuitemHide"));
            menuitem.setAttribute("value", "false");
            menuitem.setAttribute(
                "oncommand",
                "prefswitchProfileDialog.click('" + switchID + "', document.getElementById('" + switchID + "').selectedItem.getAttribute('id'), false, document.getElementById('disabled-" + switchID + "').getAttribute('checked') != 'true')");
            menupopup.appendChild(menuitem);
            menulist.appendChild(menupopup);
            hbox.appendChild(menulist);

            // CREATE DISABLE/ENABLE CHECKBOX
            var checkbox = document.createElement("checkbox");
            checkbox.setAttribute("id", "disabled-" + switchID);
            checkbox.setAttribute("label", "");
//            checkbox.setAttribute("checked", "false");
            checkbox.setAttribute("oncommand", "prefswitchProfileDialog.disableFeature('" + switchID + "')");
            hbox.appendChild(checkbox);

            // CREATE SWITCH MENULIST
            var label = document.createElement("label");
            label.setAttribute("value", mySwitch.getName() + ":");
            hbox.appendChild(label);
            menulist = document.createElement("menulist");
            menulist.setAttribute("id", switchID);
            menulist.setAttribute("name", switchID);
            menulist.setAttribute("label", mySwitch.getName());
            var menupopup = document.createElement("menupopup");
            var aState = mySwitch.getStates();
            var tState = aState.shift();
            while (tState)
            {
                var menuitem = document.createElement("menuitem");
                menuitem.setAttribute("id", tState.getID());
                menuitem.setAttribute("label", tState.getName());
                menuitem.setAttribute(
                    "oncommand",
                    "prefswitchProfileDialog.click('" + switchID + "', '" + tState.getID() + "', document.getElementById('show-" + switchID + "').selectedItem.getAttribute('value'))");
                menupopup.appendChild(menuitem);
                tState = aState.shift();
            }
            menulist.appendChild(menupopup);
            hbox.appendChild(menulist);
            this.vbox.appendChild(hbox);
            mySwitch = aSwitch.shift();
        }
    }

    this.loadProfiles = function(profileName)
    {
        var file = prefswitch.getFile(prefswitch.PROFILE_PATH, true);
        var aProfile = prefswitch.getProfiles(file);
        for (var i = 0; i < aProfile.length; i++)
        {
            var pr = aProfile[i];
            var listitem = this.listbox.appendItem(pr.getName());
            listitem.profile = pr;
            if (profileName && profileName == pr.getName()) this.listbox.selectItem(listitem);
        }
        if (this.listbox.getRowCount() > 0 && this.listbox.selectedIndex < 0) this.listbox.selectedIndex = 0;
    }

    this.refreshProfiles = function()
    {
        if (this.listbox.selectedIndex < 0)
        {
            this.listitem = null;
            this.profile = null;
            this.dialogheader.setAttribute("title", "");
        }
        else
        {
            this.listitem = this.listbox.getSelectedItem(0);
            if (!this.listitem) return;
            this.profile = this.listitem.profile;
            if (!this.profile) return;
            this.listitem.setAttribute("label", this.profile.getName());
            this.dialogheader.setAttribute("title", this.profile.getName());

            // SELECT MENUITEMS
            var aMenulist = document.getElementsByTagName("menulist");
            for (var i = 0; i < aMenulist.length; i++)
            {
                var menulist = aMenulist[i];
                if (menulist.getAttribute("id").indexOf("show-") == 0) continue;
                var switchID = menulist.getAttribute("name");
                var feature = this.profile.getFeature(switchID);

                if (!feature)
                {
//                    this.profile.setFeature(switchID, document.getElementById(switchID).selectedItem.getAttribute("id"), true, true);
                    this.profile.setFeature(switchID, document.getElementById(switchID).selectedItem.getAttribute("id"), false, true);
                    feature = this.profile.getFeature(switchID);
                }

//                if (feature)
//                {
                    if (feature.disabled)
                    {
//                        document.getElementById("show-" + switchID).setAttribute("disabled", "true");
                        document.getElementById("disabled-" + switchID).setAttribute("checked", "false");
                        document.getElementById(switchID).setAttribute("disabled", "true");
                    }
                    else
                    {
//                        document.getElementById("show-" + switchID).setAttribute("disabled", "false");
                        document.getElementById("disabled-" + switchID).setAttribute("checked", "true");
                        document.getElementById(switchID).setAttribute("disabled", "false");
                    }

                    if (feature.show) document.getElementById("show-" + switchID).selectedIndex = 0;
                    else document.getElementById("show-" + switchID).selectedIndex = 1;

                    var menupopup = document.getElementById(switchID).firstChild;
                    var menuitem = menupopup.firstChild;
                    while (menuitem)
                    {
                        if (feature.value == menuitem.getAttribute("id"))
                        {
                            document.getElementById(switchID).selectedItem = menuitem;
                            break;
                        }
                        menuitem = menuitem.nextSibling;
                    }
//                }
//                else
//                {
//                    menulist.selectedIndex = 0;
//                    this.profile.setFeature(switchID, document.getElementById(switchID).selectedItem.getAttribute("id"), true);
//                }
            }
        }
    }

    this.accept = function()
    {
        var data = this.generateXml();
        prefswitch.saveProfiles(data);
        prefswitch.setValue(prefswitch.PROFILE, "user-" + (new Date()).getTime(), prefswitch.STRING);
    }

    this.cancel = function()
    {
    }

    this.generateXml = function()
    {
        var r =
"<?xml version=\"1.0\" encoding=\"" + prefswitch.CHARSET + "\"?>\n" +
"<prefswitch>\n";
        for (var i = 0; i < this.listbox.getRowCount(); i++)
        {
            var pr = this.listbox.getItemAtIndex(i).profile;
            if (!pr) continue;
            r +=
"    <profile" +
        " id=\"" + pr.getID() + "\"" +
        " name=\"" + prefswitch.encode(pr.getName()) + "\"" +
        ">\n";
            for (var key in pr.getFeatures())
            {
                var feature = pr.getFeature(key);
                r +=
"        <feature" +
            " key=\"" + key + "\"" +
            " value=\"" + feature.value + "\"" +
            " show=\"" + feature.show + "\"" +
            " disabled=\"" + feature.disabled + "\"" +
            "/>\n";
            }
            r +=
"    </profile>\n";
        }
        r +=
"</prefswitch>";
        return r;
    }
}
