
/*
Copyright 2007-2009 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of prefswitch.

Prefswitch is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Prefswitch is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with prefswitch; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function PrefswitchSwitchDialog()
{
    this.sw = null; // switch
    this.st = null; // status
    this.pr = null; // preference

    this.listboxSwitches = null;
    this.dialogheaderSwitch = null;
    this.initial = null;
    this.listboxStates = null;
    this.dialogheaderState = null;
    this.descriptionState = null;
    this.tooltip = null;
    this.color = null;
    this.previewState = null
    this.preview = null;
    this.preference = null;
    this.type = null;
    this.visible = null;
    this.valuePref = null;
    this.valuePrefBool = null;
    this.valuePrefList = null;

    this.descriptionPreference = null;
    this.add = null;
    this.update = null;
    this.tree = null;

    this.columns =
    {
        name: [0, 0], // colName: [index, order]
        type: [1, 0],
        value: [2, 0],
        description: [3, 0]
    }

    this.moveSwitchUp = function()
    {
        var i = this.listboxSwitches.selectedIndex;
        if (i > 0)
        {
            var j = i - 1;
            this.listboxSwitches.removeEventListener("select", this, false);
            var sw = this.listboxSwitches.removeItemAt(i).sw;
            var listitem = this.listboxSwitches.insertItemAt(j, sw.getName(), "");
            listitem.sw = sw;
            this.listboxSwitches.selectedIndex = j;
            this.listboxSwitches.addEventListener("select", this, false);
        }
    }

    this.moveSwitchDown = function()
    {
        var i = this.listboxSwitches.selectedIndex;
        if (i > -1 && i < this.listboxSwitches.getRowCount() - 1)
        {
            var j = i + 1;
            this.listboxSwitches.removeEventListener("select", this, false);
            var sw = this.listboxSwitches.removeItemAt(i).sw;
            var listitem = this.listboxSwitches.insertItemAt(j, sw.getName(), "");
            listitem.sw = sw;
            this.listboxSwitches.selectedIndex = j;
            this.listboxSwitches.addEventListener("select", this, false);
        }
    }

    this.moveStateUp = function()
    {
        var i = this.listboxStates.selectedIndex;
        if (i > 0)
        {
            var j = i - 1;
            this.listboxStates.removeEventListener("select", this, false);
            var st = this.listboxStates.removeItemAt(i).st;
            var listitem = this.listboxStates.insertItemAt(j, st.getName(), "");
//            this.listboxSwitches.getSelectedItem().sw.switchStates(i, j);
            this.listboxSwitches.getItemAtIndex(this.listboxSwitches.selectedIndex).sw.switchStates(i, j);
            listitem.st = st;
            this.listboxStates.selectedIndex = j;
            this.listboxStates.addEventListener("select", this, false);
        }
    }

    this.moveStateDown = function()
    {
        var i = this.listboxStates.selectedIndex;
        if (i > -1 && i < this.listboxStates.getRowCount() - 1)
        {
            var j = i + 1;
            this.listboxStates.removeEventListener("select", this, false);
            var st = this.listboxStates.removeItemAt(i).st;
            var listitem = this.listboxStates.insertItemAt(j, st.getName(), "");
//            this.listboxSwitches.getSelectedItem().sw.switchStates(i, j);
            this.listboxSwitches.getItemAtIndex(this.listboxSwitches.selectedIndex).sw.switchStates(i, j);
            listitem.st = st;
            this.listboxStates.selectedIndex = j;
            this.listboxStates.addEventListener("select", this, false);
        }
    }

    this.newSwitch = function()
    {
        var sw = new PrefswitchSwitch(null, "New Switch");
        if (this.renameSwitch(sw))
        {
            var listitem = this.listboxSwitches.appendItem(sw.getName());
            listitem.sw = sw;
            this.listboxSwitches.selectedIndex = this.listboxSwitches.getRowCount() - 1;
        }
    }

    this.renameSwitch = function(sw)
    {
        var refresh = false;
        if (!sw)
        {
            if (this.listboxSwitches.selectedIndex < 0) return false;
            sw = this.sw;
            refresh = true;
        }
        var name = prompt("Enter name of switch:", sw.getName());
        if (!name || name == "") return false;
        sw.setName(name);
        if (refresh)
        {
            var listbox = this.listboxSwitches.getItemAtIndex(this.listboxSwitches.selectedIndex);
            listbox.label = listbox.sw.getName();
            this.refreshSwitches();
        }
        return true;
    }

    this.removeSwitch = function()
    {
        var selIndex = this.listboxSwitches.selectedIndex;
        if (selIndex < 0) return;
        this.listboxSwitches.removeItemAt(selIndex);
//        this.listboxSwitches.selectedIndex = selIndex-1;
        this.refreshSwitches();
    }

    this.newState = function()
    {
        if (this.listboxSwitches.selectedIndex < 0) return;

        var st = new PrefswitchState(null, "New State");
        if (this.renameState(st))
        {
            this.sw.addState(st);
            var listitem = this.listboxStates.appendItem(st.getName());
            listitem.st = st;
            this.listboxStates.selectedIndex = this.listboxStates.getRowCount() - 1;
        }
    }

    this.renameState = function(st)
    {
        var refresh = false;
        if (!st)
        {
            if (this.listboxStates.selectedIndex < 0) return false;
            st = this.st;
            refresh = true;
        }
        var name = prompt("Enter name of state:", st.getName());
        if (!name || name == "") return false;
        st.setName(name);
        if (refresh)
        {
            var listbox = this.listboxStates.getItemAtIndex(this.listboxStates.selectedIndex);
            listbox.label = listbox.st.getName();
            this.refreshStates(true);
        }
        return true;
    }

    this.removeState = function()
    {
        var selIndex = this.listboxStates.selectedIndex;
        if (selIndex < 0) return;
        this.listboxStates.removeItemAt(selIndex);
        this.sw.removeState(selIndex);
//        this.listboxStates.selectedIndex = selIndex-1;
        this.refreshStates(true);
    }

    this.removeAllStates = function()
    {
        if (this.listboxSwitches.selectedIndex < 0) return;
        this.sw.setStates([]);
        this.refreshStates(true);
    }

    this.removePreference = function()
    {
        if (this.tree.currentIndex < 0) return;
        this.st.removePreference(this.tree.currentIndex);
        this.refreshPreferences(true);
    }

    this.removeAllPreferences = function()
    {
        if (this.listboxStates.selectedIndex < 0) return;
        this.st.setPreferences([]);
        this.refreshPreferences(true);
    }

    this.setColor = function(color)
    {
        if (!this.sw || !this.st) return;

        if (color) this.st.setColor(color);
        else this.color.color = this.st.getColor();

        this.preview.setAttribute("style", "color:" + this.st.getColor());
        this.preview.value = this.sw.getInitial();
    }

    this.setType = function(type, value)
    {
        if (!this.pr) return;

        if (!type) type = this.pr.getType();
        if (this.pr.getType() == prefswitch.BOOLEAN && type != prefswitch.BOOLEAN) value = "";

        this.visible = this.valuePref;
        var hidden = [this.valuePrefBool, this.valuePrefList];

        var index = 0;

        switch (type)
        {
            case prefswitch.INTEGER:
                index = 1;
                break;

            case prefswitch.STRING:
                index = 2;
                break;

            case prefswitch.RESET:
                index = 3;
                this.visible.disabled = true;
                break;

            case prefswitch.LIST:
                index = 4;
                this.visible = this.valuePrefList;
                hidden = [this.valuePref, this.valuePrefBool];
                break;

            case prefswitch.LISTITEMS:
                index = 5;
                this.visible = this.valuePrefList;
                hidden = [this.valuePref, this.valuePrefBool];
                break;

            default:
                type = prefswitch.BOOLEAN;
                this.visible = this.valuePrefBool;
                hidden = [this.valuePref, this.valuePrefList];
        }

        this.type.selectedIndex = index;
        this.pr.setType(type);
        this.setValue(value, true);

        if (type != prefswitch.RESET) this.visible.disabled = false;

        for (var i in hidden) hidden[i].hidden = true;
        this.visible.hidden = false;
    }

    this.setValue = function(value, show)
    {
        if (!this.pr) return;

        if (!value && value != "")
        {
            show = true;
            value = this.pr.getValue();
        }

        var type = this.pr.getType();
        var valueSet = value;
        var valueShow = value;

        switch(type)
        {
            case prefswitch.INTEGER:
                if (!value || !value.match(/^[+-]?[0-9]+$/))
                {
                    valueSet = "";
                    valueShow = "";
                }
                break;

            case prefswitch.STRING:
                valueSet = value.replace(/[/]*\n+/g, ' ');
                valueShow = valueSet;
                break;

            case prefswitch.RESET:
                valueSet = "";
                valueShow = "";
                break;

            case prefswitch.LIST:
                valueSet = value.replace(/[/]*\n+/g, ' ').replace(/^\s+/, '').replace(/[/]*\s*$/, '');
                valueShow = value.replace(/^\s+/, '').replace(/[/]*\s*$/, '').replace(/[/]*\s+/g, "\n");
                break;

            case prefswitch.LISTITEMS:
                valueSet = value.replace(/[/]*\n+/g, ' ').replace(/^\s+/, '').replace(/[/]*\s*$/, '');
                valueShow = value.replace(/^\s+/, '').replace(/[/]*\s*$/, '').replace(/[/]*\s+/g, "\n");
                break;

            default:
                this.visible.selectedIndex = value == "false" ? 1 : 0;
        }

        this.pr.setValue(valueSet);

        if (show && type != prefswitch.BOOLEAN) this.visible.value = valueShow;
    }

    this.addPreference = function()
    {
        if (this.pr.isHidden() || this.listboxStates.selectedIndex < 0) return;
        this.st.addPreference(this.pr);
        this.st.sortPreferences();
        this.refreshPreferences(true);
    }

    this.updatePreference = function()
    {
        if (this.pr.isHidden() || this.listboxStates.selectedIndex < 0) return;
        this.st.setPreference(this.tree.currentIndex, this.pr);
        this.st.sortPreferences();
        this.refreshPreferences(true);
    }

    this.openHelp = function(url, topic, contentPackSpec)
    {
        if (this.helper) this.helper.openHelp(url, topic, contentPackSpec);
    }

    this.load = function()
    {
        var switchName = null;
        var stateName = null;
        if ("arguments" in window && window.arguments.length > 0)
        {
            if (!prefswitch) prefswitch = window.arguments[0];
            this.helper = window.arguments[1];
            switchName = window.arguments[2];
            stateName = window.arguments[3];
        }

        this.listboxSwitches = document.getElementById("listbox-switches");
        this.dialogheaderSwitch = document.getElementById("dialogheader-switch");
        this.initial = document.getElementById("initial");
        this.listboxStates = document.getElementById("listbox-states");
        this.dialogheaderState = document.getElementById("dialogheader-state");
        this.descriptionState = document.getElementById("description-state");
        this.tooltip = document.getElementById("tooltip");
        this.color = document.getElementById("color-state");
        this.previewState = document.getElementById("preview-state");
        this.preview = document.getElementById("preview");
        this.preference = document.getElementById("preference");
        this.type = document.getElementById("type");
        this.valuePref = document.getElementById("value-preference");
        this.valuePrefBool = document.getElementById("value-preference-boolean");
        this.valuePrefList = document.getElementById("value-preference-list");
        this.descriptionPreference = document.getElementById("description-preference");
        this.add = document.getElementById("add-preference");
        this.update = document.getElementById("update-preference");
        this.tree = document.getElementById("tree");
        this.listboxSwitches.addEventListener("select", this, false);
        this.listboxStates.addEventListener("select", this, false);
        this.tree.addEventListener("select", this, false);
        this.refreshSwitches(true, switchName, stateName);
    }

    this.handleEvent = function(event)
    {
        if (!event) return;

        switch(event.target.id)
        {
            case "listbox-switches": this.refreshSwitches(false); break;
            case "listbox-states": this.refreshStates(false); break;
            case "tree": this.refreshPreferences(false);
        }
    }

    this.refreshSwitches = function(load, switchName, stateName)
    {
try{
        if (load)
        {
            this.listboxSwitches.removeEventListener("select", this, false);
            while (this.listboxSwitches.getRowCount() > 0 && this.listboxSwitches.removeItemAt(0));
            this.listboxSwitches.addEventListener("select", this, false);
            var file = prefswitch.getFile(prefswitch.SWITCH_PATH, true);
            var aSwitch = prefswitch.getSwitches(file);
            for (var i = 0; i < aSwitch.length; i++)
            {
                var sw = aSwitch[i];
                var listitem = this.listboxSwitches.appendItem(sw.getName());
                listitem.sw = sw;
                if (switchName && switchName == sw.getName()) this.listboxSwitches.selectItem(listitem);
            }
        }

        if (/*load ||*/ this.listboxSwitches.selectedIndex < 0) this.sw = new PrefswitchSwitch();
        else this.sw = this.listboxSwitches.getSelectedItem(0).sw;

//        if (this.sw)
//        {
            this.dialogheaderSwitch.setAttribute("title", this.sw.getName());
            this.initial.value = this.sw.getInitial();
//        }

        this.refreshStates(true, stateName);
}catch(e){alert("Switches:"+e);}
    }

    this.refreshStates = function(load, stateName)
    {
try{
        if (load)
        {
            this.listboxStates.removeEventListener("select", this, false);
            while (this.listboxStates.getRowCount() > 0 && this.listboxStates.removeItemAt(0));
            this.listboxStates.addEventListener("select", this, false);
            var aStates = this.sw.getStates();
            for (var i = 0; i < aStates.length; i++)
            {
                var st = aStates[i];
                var listitem = this.listboxStates.appendItem(st.getName());
                listitem.st = st;
                if (stateName && stateName == st.getName()) this.listboxStates.selectItem(listitem);
            }
        }

        if (/*load ||*/ this.listboxStates.selectedIndex < 0)
        {
            this.st = new PrefswitchState();
            this.previewState.hidden = true;
            this.preview.hidden = true;
        }
        else
        {
            this.st = this.listboxStates.getSelectedItem(0).st;
            this.previewState.hidden = false;
            this.preview.hidden = false;
        }

//        if (this.st)
//        {
            this.dialogheaderState.setAttribute("title", this.st.getName());
            this.descriptionState.value = this.st.getDescription();
            this.tooltip.value = this.st.getTooltip();
            this.setColor();
//        }

        this.refreshPreferences(true);
}catch(e){alert("States:"+e);}
    }

    this.refreshPreferences = function(load)
    {
try{
        if (load)
        {
            var aPreferences = this.st ? this.st.getPreferencesArray() : [];
            this.tree.view = new PrefswitchTreeView(aPreferences, this.columns, aPreferences.length);
        }

        if (load || this.tree.currentIndex < 0)
        {
            this.pr = new PrefswitchPreference();
            this.setType("boolean", "true");
            this.update.hidden = true;
        }
        else
        {
            this.pr = this.st.getPreference(this.tree.currentIndex).createClone();
            this.update.hidden = false;
        }

//        if (this.pr)
//        {
            this.preference.value = this.pr.getName();
            this.setType();
            this.descriptionPreference.value = this.pr.getDescription();
//        }
}catch(e){alert("Preferences:"+e);}
    }

    this.sortColumn = function(column)
    {
        var order = this.columns[column.id][1];
        this.columns[column.id][1] = order == 0 ? 1 : 0;
        this.tableSort(this.sw.getStates(), this.columns[column.id][0], order);
        this.tree.view = new PrefswitchTreeView(this.sw.getStates(), this.columns, this.sw.getStates().length);
    }

    this.tableSort = function(table, col, order)
    {
        var columnSort;
        if (order == 0) columnSort = function(a, b) { return a[col] < b[col] ? -1 : a[col] > b[col] ? 1 : 0; } // ascending
        else columnSort = function(a, b) { return a[col] > b[col] ? -1 : a[col] < b[col] ? 1 : 0; } // descending
        table.sort(columnSort);
    }

    this.accept = function()
    {
        var data = this.generateXml();
        prefswitch.saveSwitches(data);
        prefswitch.setValue(prefswitch.UPDATE, (new Date()).getTime(), prefswitch.STRING);
    }

    this.cancel = function()
    {
    }

    this.generateXml = function()
    {
        var obj = {}
        var r =
"<?xml version=\"1.0\" encoding=\"" + prefswitch.CHARSET + "\"?>\n" +
"<prefswitch>\n";
        var rowCount = this.listboxSwitches.getRowCount();
        for (var i=0; i < rowCount; i++)
        {
            var sw = this.listboxSwitches.getItemAtIndex(i).sw;
            r +=
"    <switch" +
        " id=\"" + sw.getID() + "\"" +
        " name=\"" + prefswitch.encode(sw.getName()) + "\"" +
        " initial=\"" + prefswitch.encode(sw.getInitial()) + "\"" +
        ">\n";
            var statesLength = sw.getStates().length;
            for (var j=0; j < statesLength; j++)
            {
                var st = sw.getState(j);
                r +=
"        <state" +
            " id=\"" + st.getID() + "\"" +
            " name=\"" + prefswitch.encode(st.getName()) + "\"";
                if (st.getDescription() != "") r +=
            " description=\"" + prefswitch.encode(st.getDescription()) + "\"";
                if (st.getTooltip() != "") r +=
            " tooltip=\"" + prefswitch.encode(st.getTooltip()) + "\"";
                if (st.getColor() != "") r +=
            " color=\"" + prefswitch.encode(st.getColor()) + "\"";
                r +=
            ">\n";
                var referencesLength = st.getPreferences().length;
                for (var k=0; k < referencesLength; k++)
                {
                    var pr = st.getPreference(k);

                    // PURGE LISTITEMS PREFERENCE
                    if (pr.getType() == prefswitch.LISTITEMS && !obj[pr.getName()])
                    {
                        obj[pr.getName()] = true;
                        prefswitch.setValue(pr.getName(), "", prefswitch.STRING);
                    }

                    r +=
"            <preference" +
                " name=\"" + prefswitch.encode(pr.getName()) + "\"" +
                " type=\"" + pr.getType() + "\"";
                    if (pr.getType() != prefswitch.RESET) r +=
                " value=\"" + prefswitch.encode(pr.getValue()) + "\"";
                    if (pr.getDescription() != "") r +=
                " description=\"" + prefswitch.encode(pr.getDescription()) + "\"";
                    r +=
                "/>\n";
                }
                r +=
"        </state>\n";
            }
            r +=
"    </switch>\n";
        }
        r +=
"</prefswitch>";
        return r;
    }
}

function PrefswitchTreeView(table, columns, rowcount)
{
    this.table = table ? table : null;
    this.columns = columns ? columns : null;
    this.rowCount = rowcount ? rowcount : 0;
    this.cycleHeader = function(column) { /*sortColumn(column);*/ }
    this.getCellProperties = function(row, col, props) {}
    this.getCellText = function(row, column) { return this.table[row][this.columns[column.id][0]]; }
    this.getColumnProperties = function(colid, col, props) {}
    this.getImageSrc = function(row, col) { return null; }
    this.getLevel = function(row) { return 0; }
    this.getRowProperties = function(row, props) {}
    this.isContainer = function(row) { return false; }
    this.isSeparator = function(row) { return false; }
    this.isSorted = function(row) { return false; }
    this.setTree = function(treebox) { this.treebox=treebox; }
}
